
import math
from math import radians, cos, sin
from ezdxf import colors
import ezdxf
from ezdxf.math import Vec2
from ezdxf.render import mleader
from fractions import Fraction

from createdrawing.common import *

# Section View START

def draw_flow_indicator(msp, base_x=0, base_y=0, width=1, height=2, tail_length=6, layer='FLOW-INDICATOR', direction='down'):
    if direction == 'up':
        triangle_points = [
            (base_x, base_y), 
            (base_x + width, base_y), 
            (base_x + width / 2, base_y + height), 
            (base_x, base_y), 
        ]
        tail_points = [
            (base_x + width / 2, base_y), 
            (base_x + width / 2, base_y - 2), 
            (base_x - width / 4, base_y - 2.5), 
            (base_x + width, base_y - 2.8), 
            (base_x + width / 2, base_y - 3), 
            (base_x + width / 2, base_y - tail_length), 
        ]
    elif direction == 'down':
        triangle_points = [
            (base_x, base_y),
            (base_x + width, base_y),
            (base_x + width / 2, base_y - height),
            (base_x, base_y),
        ]
        tail_points = [
            (base_x + width / 2, base_y),  
            (base_x + width / 2, base_y + 2),  
            (base_x - width / 4, base_y + 2.5),  
            (base_x + width, base_y + 2.8),  
            (base_x + width / 2, base_y + 3),   
            (base_x + width / 2, base_y + tail_length), 
        ]

    msp.add_lwpolyline(triangle_points, close=True, dxfattribs={'layer': layer})
    
    msp.add_lwpolyline(tail_points, close=False, dxfattribs={'layer': layer})
    
    hatch = msp.add_hatch(color=colors.RED)
    hatch.paths.add_polyline_path(triangle_points, is_closed=True)

def parse_mixed_fraction(mixed_str):
    if ' ' in mixed_str:
        whole_number, fraction = mixed_str.split(' ')
        numerator, denominator = fraction.split('/')
        return int(whole_number) * 12 + Fraction(int(numerator), int(denominator))
    elif '/' in mixed_str:
        numerator, denominator = mixed_str.split('/')
        return Fraction(int(numerator), int(denominator))
    else:
        return int(mixed_str)

def inches_to_feet_and_inches(mixed_str):
    inches = parse_mixed_fraction(mixed_str)
    feet = int(inches // 1) // 12
    remainder_inches = inches - feet * 12
    if remainder_inches == 0:
        return f"{feet}'-0\""
    else:
        return f"{feet}'-{remainder_inches:.0f}\""

def draw_accessories_tower_cap(msp, x_offset=0, y_offset=0, tower_width=0, tower_height=0, layer='HOOD-ACCESSORIES'):
    cap_points = [
        (x_offset, y_offset + tower_height),                           
        (x_offset + (- 0.3 * tower_width), y_offset + tower_height),       
        (x_offset + (0.3 * tower_width), y_offset + tower_height),       
        (x_offset + (0.45 * tower_width), y_offset + tower_height - 0.3 * tower_width),  
        (x_offset + (0.65 * tower_width), y_offset + tower_height + 0.02 * tower_height),
        (x_offset + (0.7 * tower_width), y_offset + tower_height),   
        (x_offset + (1.3 * tower_width), y_offset + tower_height)        
    ]
    msp.add_lwpolyline(cap_points, dxfattribs={'layer': layer})

def draw_hood_top_collar(msp, x_offset=0, y_offset=0, length=26, height=3, width=10, layer='HOOD-ACCESSORIES', flow_indicator=True):
    points = [
        (x_offset, y_offset),
        (x_offset + length, y_offset),
        (x_offset + length, y_offset + height),
        (x_offset, y_offset + height),
    ]
    flow_indicator_x_offset = x_offset + length / 2
    flow_indicator_y_offset = y_offset + height * 2.5 if layer == 'MUA-DUCT' else y_offset + height * 4
    flow_indicator_direction = 'down' if layer == 'MUA-DUCT' else 'up'
    if flow_indicator:
        draw_flow_indicator(msp, flow_indicator_x_offset, flow_indicator_y_offset, width=1, height=2, tail_length=6, layer='FLOW-INDICATOR', direction=flow_indicator_direction)
        if(layer == 'MUA-DUCT'):
            msp.add_mtext(f"{layer} \n {length}\"X{width}\"", dxfattribs={'insert': (x_offset + length / 2 - 10, y_offset + height * 10.5), 'color': colors.YELLOW, 'char_height': 1.5})
        else:
            msp.add_mtext(f"{layer} \n {length}\"X{width}\"", dxfattribs={'insert': (x_offset + length / 2 - 10, y_offset + height * 7.5), 'color': colors.YELLOW, 'char_height': 1.5})
    msp.add_lwpolyline(points, close=True, dxfattribs={'layer': layer})

def draw_section_view_tool(msp, x_offset=0, y_offset=0, hood_height=30, layer='HOOD-ACCESSORIES'):
    box_points = [
        (x_offset + 1.5, y_offset + 2),
        (x_offset + 1.5, y_offset - 0.25 + 2),
        (x_offset + 4.5, y_offset - 0.25 + 2),
        (x_offset + 4.5, y_offset - 2 + 2),
        (x_offset + 4.8, y_offset - 2 + 2),
        (x_offset + 4.8, y_offset + 2),
        (x_offset + 1.5, y_offset + 2),
        
    ]
    line_points = [
        (x_offset + 1.5, y_offset + 1.75),
        (x_offset + 5, y_offset + 1.75)
    ]
    
    tool_points = [
        (x_offset + 2.25, y_offset + 1.75),
        (x_offset + 2.25, y_offset + 1.5),
        
        (x_offset + 2.25, y_offset + 1.75),
        
        (x_offset + 2.70, y_offset + 1.75),
        (x_offset + 2.70, y_offset + 1.5),
        
        (x_offset + 2.70, y_offset + 1.75),
        
        (x_offset + 3.55, y_offset + 1.75),
        (x_offset + 3.55, y_offset + 1.5),
        
        (x_offset + 3.55, y_offset + 1.75),
        
        (x_offset + 3.98, y_offset + 1.75),
        (x_offset + 3.98, y_offset + 1.5),
    ]
    
    msp.add_lwpolyline(box_points, close=False, dxfattribs={'layer': layer})
    # msp.add_lwpolyline(line_points, close=True, dxfattribs={'layer': layer})
    msp.add_lwpolyline(tool_points, close=False, dxfattribs={'layer': layer})
    
    
    arc_points = [
        ((x_offset + 2.25, y_offset + 1.5, math.tan(math.radians(25))), (x_offset + 2.70, y_offset + 1.5, 0)),
        ((x_offset + 2.70, y_offset + 1.5, math.tan(math.radians(14))), (x_offset + 3.55, y_offset + 1.5, 0)),
        ((x_offset + 3.55, y_offset + 1.5, math.tan(math.radians(26))), (x_offset + 3.98, y_offset + 1.5, 0))
    ]

    for start, end in arc_points:
        points = [start, end]
        msp.add_lwpolyline(points, format='xyb', dxfattribs={'layer': layer})
        
    
    arc_line_points = [
        (x_offset + 2.25, y_offset + 1.395),
        (x_offset + 2.70, y_offset + 1.395),
        (x_offset + 3.55, y_offset + 1.395),
        (x_offset + 3.98, y_offset + 1.395),
    ]
    
    for index, (start, end) in enumerate(zip(arc_line_points, arc_line_points[1:])):
        if index == 0:
            start = ((start[0] + end[0]) / 2, start[1]) 
        elif index == 2:
            end = ((start[0] + end[0]) / 2, end[1]) 

        points = [start, end]
        msp.add_lwpolyline(points, close=False, dxfattribs={'layer': layer})
    
    arc_bottom_tool_points = [
        (x_offset + 2.8, y_offset + 1.395),
        (x_offset + 2.8, y_offset + 0.9),
        (x_offset + 2.88, y_offset + 0.82),
        (x_offset + 3.3, y_offset + 0.82),
        
        (x_offset + 3.4, y_offset + 0.9),
        (x_offset + 2.8, y_offset + 0.9),
        
        (x_offset + 3.4, y_offset + 0.9),
        (x_offset + 3.4, y_offset + 1.395),
    ]
    
    for index, (start, end) in enumerate(zip(arc_bottom_tool_points, arc_bottom_tool_points[1:])):
        points = [start, end]
        msp.add_lwpolyline(points, close=False, dxfattribs={'layer': layer})
    
    
    accessory_tower_points = [
        (x_offset + 2.8, y_offset + 1.75),
        (x_offset + 2.8, y_offset + 5.5),
        
        (x_offset + 2.8, y_offset + 1.75),
        
        (x_offset + 3.4, y_offset + 1.75),
        (x_offset + 3.4, y_offset + 5.5),
    ]
    msp.add_lwpolyline(accessory_tower_points, close=False, dxfattribs={'layer': layer})
    
    draw_accessories_tower_cap(msp, x_offset=x_offset + 2.8, y_offset=y_offset, tower_width=0.6, tower_height=5.5, layer=layer)
    

def draw_section_view_hood_collars(msp, x_offset=0, y_offset=0, hood_height=30, hood_length=54, collars={},layer='HOOD-ACCESSORIES'):
    exh_collar = collars['exh_collar']
    mua_collar = collars['mua_collar']
    
    collar_configs = []
    
    config = {
        'x_offset': x_offset + hood_length + 3, 
        'layer': 'MUA-DUCT',
        'length': mua_collar['MUACOLLARWIDTH'],
        'width': mua_collar['MUACOLLARWIDTH']
    }
    collar_configs.append(config)
    
    config = {
        'x_offset': x_offset + 5.5, 
        'layer': 'EXHAUST-DUCT',
        'length': exh_collar['EXHCOLLARWIDTH'],
        'width': exh_collar['EXHCOLLARWIDTH']
    }
    collar_configs.append(config)
    
    for config in collar_configs:
        draw_hood_top_collar(msp, x_offset=config['x_offset'], y_offset=y_offset, length=config['length'], height=3, width=config['width'], layer=config['layer'], flow_indicator=False)
    

def draw_section_view_hood_top(msp, x_offset=0, y_offset=0, hood_height=30, hood_length=54, collars={},layer='HOOD-ACCESSORIES'):
    
    draw_section_view_tool(msp, x_offset=x_offset + 1.5, y_offset=y_offset, hood_height=hood_height, layer=layer)
    draw_section_view_tool(msp, x_offset=x_offset + hood_length - 8, y_offset=y_offset, hood_height=hood_height, layer=layer)
    
    draw_section_view_hood_collars(msp, x_offset=x_offset, y_offset=y_offset, hood_height=hood_height, hood_length=hood_length, collars=collars, layer=layer)
    
    

def draw_section_view_hood(msp, x_offset=0, y_offset=0, hood_length=54, hood_height=30, plenum={}, hood_info={}, collar={}):
    
    hood_no = hood_info['HOOD_NO']
    
    points = [
        (x_offset, y_offset),
        (x_offset, y_offset - hood_height),
        (x_offset + hood_length, y_offset - hood_height),
        (x_offset + hood_length, y_offset),
        (x_offset, y_offset),
    ]
    
    msp.add_text(
        f"HOOD #{hood_no}", 
        dxfattribs={
            'layer': 'HOOD-ACCESSORIES',
            'insert': (x_offset + hood_length * 0.15, y_offset - hood_height * 0.85),
        }
    )
    
    msp.add_lwpolyline(
        points,
        close=False, dxfattribs={'layer': 'HOOD'}
    )
    
    draw_section_view_hood_top(msp, x_offset=x_offset, y_offset=y_offset, hood_height=hood_height, hood_length=hood_length, collars=collar, layer='HOOD-ACCESSORIES')
    
    points = [
        (x_offset + 3, y_offset),
        (x_offset + 3, y_offset - hood_height * 0.5),
        (x_offset + 3 + 1.5, y_offset - hood_height * 0.5),
        (x_offset, y_offset - hood_height * 0.6 - 3),
        (x_offset + 4.5, y_offset - hood_height * 0.6 + 3),
        (x_offset + 4.5 + 1.4, y_offset - hood_height * 0.6 + 3 + 1.9),
        (x_offset + 4.5 + 1.4 - 0.15, y_offset - hood_height * 0.6 + 3 + 1.9 + 0.10),
        (x_offset + 4.5 + 1.4 - 0.15 - 0.4, y_offset - hood_height * 0.6 + 3 + 1.9 + 0.10 - 0.5),
    ]
    
    msp.add_lwpolyline(points, close=False, dxfattribs={'layer': 'HOOD'})
    
    rectangle_points = [
        (x_offset + 4.5 + 1.4 - 0.15, y_offset - hood_height * 0.6 + 3 + 1.9 + 0.10),
        (x_offset + 4.5 + 1.4 - 0.15 - 1, y_offset - hood_height * 0.6 + 3 + 1.9 + 0.10 + 1.2),
        (x_offset + 4.5 + 1.4 - 0.15 - 1 + 10 - 0.5, y_offset - hood_height * 0.6 + 3 + 1.9 + 0.10 + 1.2 + 9),
        (x_offset + 4.5 + 1.4 - 0.15 - 1 + 10 - 0.5 + 1, y_offset - hood_height * 0.6 + 3 + 1.9 + 0.10 + 1.2 + 9 - 1),
        (x_offset + 4.5 + 1.4 - 0.15, y_offset - hood_height * 0.6 + 3 + 1.9 + 0.10),
    ]
    
    msp.add_lwpolyline(rectangle_points, close=False, dxfattribs={'layer': 'HOOD-ACCESSORIES'})
    
    hatch = msp.add_hatch(color=colors.YELLOW)
    hatch.set_pattern_fill("ANSI31", scale=0.15, color=colors.YELLOW, angle=45)
    hatch.paths.add_polyline_path(rectangle_points, is_closed=True)
    
    exh_collar_width = collar['exh_collar']['EXHCOLLARWIDTH'];
    
    cover_points = [
        (x_offset + 5.5 + exh_collar_width - 3, y_offset - 3),
        (x_offset + 5.5 + exh_collar_width, y_offset),
        (x_offset + 5.5 + exh_collar_width + 0.7, y_offset),
        (x_offset + 5.5 + exh_collar_width + 0.7, y_offset - 3),
        (x_offset + 5.5 + exh_collar_width + 0.7 - 1.5, y_offset - 3 - 1.5),
        (x_offset + 5.5 + exh_collar_width + 0.7 - 1.5, y_offset - 3 - 1.5),
        (x_offset + 5.5 + exh_collar_width + 0.7 - 1.5 - 0.1, y_offset - 3 - 1.5 + 0.09),
    ]
    
    msp.add_lwpolyline(cover_points, close=False, dxfattribs={'layer': 'HOOD'})
    
    ceiling_rectangle_points = [
        (x_offset + 5.5 + exh_collar_width + 0.7, y_offset),
        (x_offset + 5.5 + exh_collar_width + 0.7, y_offset - 1.5),
        (x_offset + hood_length, y_offset - 1.5),
    ]
    
    msp.add_lwpolyline(ceiling_rectangle_points, close=False, dxfattribs={'layer': 'HOOD'})
    
    heat_censor_points = [
        (x_offset + 5.5 + exh_collar_width + 0.7 + 10, y_offset + 0.5),
        (x_offset + 5.5 + exh_collar_width + 0.7 + 10, y_offset + 0.5 - 2),
        (x_offset + 5.5 + exh_collar_width + 0.7 + 10 + 2, y_offset + 0.5 - 2),
        (x_offset + 5.5 + exh_collar_width + 0.7 + 10 + 2, y_offset + 0.5),
    ]
    
    heat_censor_adapter_points = [
        (x_offset + 5.5 + exh_collar_width + 0.7 + 10 + 0.6, y_offset + 0.5 - 2),
        (x_offset + 5.5 + exh_collar_width + 0.7 + 10 + 1.4, y_offset + 0.5 - 2),
        (x_offset + 5.5 + exh_collar_width + 0.7 + 10 + 1.4, y_offset + 0.5 - 2 - 0.6),
        (x_offset + 5.5 + exh_collar_width + 0.7 + 10 + 0.6, y_offset + 0.5 - 2 - 0.6),
    ]
    
    heat_censor_adapter_bar_points = [
        (x_offset + 5.5 + exh_collar_width + 0.7 + 10 + 0.6 + 0.1, y_offset + 0.5 - 2 - 0.6),
        (x_offset + 5.5 + exh_collar_width + 0.7 + 10 + 0.6 + 0.1, y_offset + 0.5 - 2 - 0.6 - 4),
        (x_offset + 5.5 + exh_collar_width + 0.7 + 10 + 0.6 + 0.1 + 0.5, y_offset + 0.5 - 2 - 0.6 - 4),
        (x_offset + 5.5 + exh_collar_width + 0.7 + 10 + 0.6 + 0.1 + 0.5, y_offset + 0.5 - 2 - 0.6),
        (x_offset + 5.5 + exh_collar_width + 0.7 + 10 + 0.6 + 0.1, y_offset + 0.5 - 2 - 0.6),
    ]
    
    msp.add_lwpolyline(heat_censor_points, close=True, dxfattribs={'layer': 'HOOD ELECTRICAL'})
    msp.add_lwpolyline(heat_censor_adapter_points, close=True, dxfattribs={'layer': 'HOOD ELECTRICAL'})
    msp.add_lwpolyline(heat_censor_adapter_bar_points, close=True, dxfattribs={'layer': 'HOOD ELECTRICAL'})
    
    
    light_base_points = [
        (x_offset + 5.5 + exh_collar_width + 0.7 + 10 + 3.5, y_offset + 0.5),
        (x_offset + 5.5 + exh_collar_width + 0.7 + 10 + 3.5, y_offset + 0.5 - 2),
        (x_offset + 5.5 + exh_collar_width + 0.7 + 10 + 3.5 + 4, y_offset + 0.5 - 2),
        (x_offset + 5.5 + exh_collar_width + 0.7 + 10 + 3.5 + 4, y_offset + 0.5),
    ]
    
    light_adapter_points = [
        (x_offset + 5.5 + exh_collar_width + 0.7 + 10 + 3.5 - 0.5, y_offset + 0.5 - 2),
        (x_offset + 5.5 + exh_collar_width + 0.7 + 10 + 3.5, y_offset + 0.5 - 2 - 1),
        (x_offset + 5.5 + exh_collar_width + 0.7 + 10 + 3.5 + 4, y_offset + 0.5 - 2 - 1),
        (x_offset + 5.5 + exh_collar_width + 0.7 + 10 + 3.5 + 4 + 0.5, y_offset + 0.5 - 2),
    ]
    
    light_sides_points = [
        (x_offset + 5.5 + exh_collar_width + 0.7 + 10 + 3.5, y_offset + 0.5 - 2 - 1 - 4.5),
        (x_offset + 5.5 + exh_collar_width + 0.7 + 10 + 3.5, y_offset + 0.5 - 2 - 1),
        (x_offset + 5.5 + exh_collar_width + 0.7 + 10 + 3.5 + 4, y_offset + 0.5 - 2 - 1),
        (x_offset + 5.5 + exh_collar_width + 0.7 + 10 + 3.5 + 4, y_offset + 0.5 - 2 - 1 - 4.5),
    ]
    
    light_inside_line1 = [
        (x_offset + 5.5 + exh_collar_width + 0.7 + 10 + 3.5 + 1.6, y_offset + 0.5 - 2 - 1 - 0.5),
        (x_offset + 5.5 + exh_collar_width + 0.7 + 10 + 3.5 + 1, y_offset + 0.5 - 2 - 1 - 1),
    ]
    
    light_inside_line2 = [
        (x_offset + 5.5 + exh_collar_width + 0.7 + 10 + 3.5 + 1.6 + 0.8, y_offset + 0.5 - 2 - 1 - 0.5),
        (x_offset + 5.5 + exh_collar_width + 0.7 + 10 + 3.5 + 1.6 - 1 , y_offset + 0.5 - 2 - 1 - 4),
    ]
    
    light_inside_line3 = [
        (x_offset + 5.5 + exh_collar_width + 0.7 + 10 + 3.5 + 1.6 + 0.8 + 0.7, y_offset + 0.5 - 2 - 1 - 0.5 - 3),
        (x_offset + 5.5 + exh_collar_width + 0.7 + 10 + 3.5 + 1.6 + 0.8, y_offset + 0.5 - 2 - 1 - 1 - 0.5 - 3 - 0.5),
    ]
    
    msp.add_lwpolyline(light_inside_line1, close=True, dxfattribs={'layer': 'HOOD ELECTRICAL'})
    msp.add_lwpolyline(light_inside_line2, close=True, dxfattribs={'layer': 'HOOD ELECTRICAL'})
    msp.add_lwpolyline(light_inside_line3, close=True, dxfattribs={'layer': 'HOOD ELECTRICAL'})
    
    light_arc_points = [
        (x_offset + 5.5 + exh_collar_width + 0.7 + 10 + 3.5, y_offset + 0.5 - 2 - 1 - 4.5, 1),
        (x_offset + 5.5 + exh_collar_width + 0.7 + 10 + 3.5 + 4, y_offset + 0.5 - 2 - 1 - 4.5, 1),
    ]
    
    msp.add_lwpolyline(light_base_points, close=True, dxfattribs={'layer': 'HOOD ELECTRICAL'})
    msp.add_lwpolyline(light_adapter_points, close=True, dxfattribs={'layer': 'HOOD ELECTRICAL'})
    msp.add_lwpolyline(light_sides_points, close=False, dxfattribs={'layer': 'HOOD ELECTRICAL'})
    msp.add_lwpolyline(light_arc_points, close=False, format='xyb', dxfattribs={'layer': 'HOOD ELECTRICAL'})

def draw_section_view_plenum(msp, x_offset=0, y_offset=0, plenum_depth=0, plenum_length=0, layer='HOOD'):
    
    points = [
        (x_offset, y_offset),
        (x_offset, y_offset - plenum_length),
        (x_offset + plenum_depth, y_offset - plenum_length),
        (x_offset + plenum_depth, y_offset),
        (x_offset, y_offset),
    ]
    
    indicators = int(plenum_depth / 11)
    
    for i in range(2):
        draw_flow_indicator(msp, (x_offset + 2) + (i * 11) , y_offset - plenum_length * 2, width=1, height=2, tail_length=6, layer='FLOW-INDICATOR')
    
    msp.add_lwpolyline(
        points,
        close=False, dxfattribs={'layer': layer}
    )
    
    msp.add_mtext(f"PERF", dxfattribs={'insert': (x_offset + plenum_depth / 3, y_offset - plenum_length / 3), 'color': colors.YELLOW, 'char_height': 2})
    
    plenum_bottom_box_points = [
        (x_offset + 1, y_offset - plenum_length + 1),
        (x_offset + plenum_depth - 1, y_offset - plenum_length + 1),
        (x_offset + plenum_depth - 1, y_offset - plenum_length + 1 -1),
        (x_offset + 1, y_offset - plenum_length + 1 - 1),
    ]
    
    msp.add_lwpolyline(plenum_bottom_box_points, close=True, dxfattribs={'layer': layer})
    
    
def draw_right_arrow(msp, base_x=0, base_y=0, end_x=0, end_y=0, width=1, height=2, layer='INDICATOR', direction='right'):
    
    triangle_points = [
        (end_x, end_y), 
        (end_x, end_y + height), 
        (end_x + width, end_y + height / 2), 
        (end_x, end_y), 
    ]
    
    tail_points = [
        (base_x, base_y + height / 2),
        (end_x, end_y + height / 2),
    ]
    
    msp.add_lwpolyline(triangle_points, close=True, dxfattribs={'layer': layer})
    
    msp.add_lwpolyline(tail_points, close=False, dxfattribs={'layer': layer})
    
    hatch = msp.add_hatch(color=colors.CYAN)
    hatch.paths.add_polyline_path(triangle_points, is_closed=True)


def draw_section_view_appliance(msp, x_offset=0, y_offset=0, layer='APPLIANCE'):
    body_points = [
        (x_offset, y_offset),
        (x_offset, y_offset + 50),
        (x_offset + 3.5, y_offset + 50),
        (x_offset + 3.5, y_offset + 30),
        (x_offset, y_offset + 30),
        (x_offset + 31.5, y_offset + 30),
        (x_offset + 31.5, y_offset + 30 + 1.4),
        (x_offset + 3.5, y_offset + 30 + 1.4),
        (x_offset + 27, y_offset + 30 + 1.4),
        (x_offset + 27, y_offset + 30),
        (x_offset + 27 + 2, y_offset + 30),
        (x_offset + 27 + 2, y_offset),
    ]
    
    msp.add_lwpolyline(
        body_points,
        close=True, dxfattribs={'layer': layer}
    )
    
    top_threshholder_points = [
        (x_offset + 3.5, y_offset + 49),
        (x_offset + 3.5 + 10, y_offset + 49),
        (x_offset + 3.5 + 10, y_offset + 49 - 0.8),
        (x_offset + 3.5 + 10 - 0.4, y_offset + 49 - 0.8 - 0.4),
        (x_offset + 3.5, y_offset + 49 - 0.8 - 0.4 - 0.8),
    ]
    
    msp.add_lwpolyline(
        top_threshholder_points,
        close=True, dxfattribs={'layer': layer}
    )
    
    body_door_points = [
        (x_offset + 29, y_offset + 5),
        (x_offset + 29 + 14, y_offset + 5),
    ]
    
    msp.add_lwpolyline(
        body_door_points,
        close=True, dxfattribs={'layer': layer}
    )
    
    body_door_arc_points = [
        (x_offset + 29 + 14, y_offset + 5, 0.5),
        (x_offset + 29, y_offset + 5 + 15, 0),
    ]
    
    msp.add_lwpolyline(body_door_arc_points, format='xyb', dxfattribs={'layer': layer})
    
    body_door_handle_points = [
        (x_offset + 29, y_offset + 5 + 15 - 0.1),
        (x_offset + 29 + 3, y_offset + 5 + 15 - 0.1 - 0.3),
        (x_offset + 29 + 3 + 0.5, y_offset + 5 + 15 - 0.1 - 0.3 - 0.5),
        (x_offset + 29 + 3 + 0.5, y_offset + 5 + 15 - 0.1 - 0.3 - 0.5 - 1),
        (x_offset + 29 + 3, y_offset + 5 + 15 - 0.1 - 0.3 - 0.5 - 1 - 0.5),
        (x_offset + 29, y_offset + 5 + 15 - 0.1 - 3),
    ]
    
    msp.add_lwpolyline(body_door_handle_points, close=True, dxfattribs={'layer': layer})
    
    foot1_points = [
        (x_offset + 2, y_offset),
        (x_offset + 2 + 0.5, y_offset - 5),
        (x_offset + 2 + 1.2, y_offset - 5),
        (x_offset + 2 + 1.2 + 0.5, y_offset),
        
    ]
    
    msp.add_lwpolyline(
        foot1_points,
        close=True, dxfattribs={'layer': layer}
    )
    
    foot2_points = [
        (x_offset + 29 - 2 , y_offset),
        (x_offset + 29 - 2 - 0.5, y_offset - 5),
        (x_offset + 29 - 2 - 1.2, y_offset - 5),
        (x_offset + 29 - 2 - 1.2 - 0.5, y_offset),
        
    ]
    
    msp.add_lwpolyline(
        foot2_points,
        close=True, dxfattribs={'layer': layer}
    )
    
    foot_nail1_points = [
        (x_offset + 2.6, y_offset - 5 - 1),
        (x_offset + 2.6, y_offset - 5),
        (x_offset + 2.6 + 0.5, y_offset - 5),
        (x_offset + 2.6 + 0.5, y_offset - 5 - 1),
        
    ]
    
    msp.add_lwpolyline(foot_nail1_points, close=True, dxfattribs={'layer': layer})
    
    foot_nail2_points = [
        (x_offset + 29 - 2 - 1.2 - 0.5 + 0.6, y_offset - 5 - 1),
        (x_offset + 29 - 2 - 1.2 - 0.5 + 0.6, y_offset - 5),
        (x_offset + 29 - 2 - 1.2 - 0.5 + 0.6 + 0.5, y_offset - 5),
        (x_offset + 29 - 2 - 1.2 - 0.5 + 0.6 + 0.5, y_offset - 5 - 1),
        
    ]
    
    msp.add_lwpolyline(foot_nail2_points, close=True, dxfattribs={'layer': layer})
    
    dim = msp.add_linear_dim(
        base=(x_offset, y_offset + 50),
        p1=(x_offset + 30, y_offset + 35),
        p2=(x_offset + 51, y_offset + 75),
        dimstyle=layer,
        override={
            "dimtsz": 0,
            "dimasz": 1,
            "dimdle": 2,
            "dimexe": 2,
            "dimblk": "ARCHTICK",
            "dimtxt": 2,
            "dimclrd": 2,  # Dimension line color (2 corresponds to yellow in AutoCAD color index)
            "dimclre": 2,  # Extension line color
            "dimclrt": 2,  # Text color
        },
        angle=0,
        text=f"6\" MIN",
    )
    
    dim.render()
    
    

def draw_section_view_info_label(msp, text='', start_x_offset=0, start_y_offset=0, pointer_x_offset=0, pointer_y_offset=0):
    ml_builder = msp.add_multileader_mtext("Standard")
    ml_builder.set_content(
        text,
        alignment= mleader.TextAlignment.right,
        color=colors.YELLOW,
        char_height=2.0,
    )
    ml_builder.add_leader_line(mleader.ConnectionSide.right, [Vec2(pointer_x_offset, pointer_y_offset)])
    ml_builder.set_arrow_properties(size=2.0)
    ml_builder.set_leader_properties(
        color=colors.CYAN,
        # linetype="DASHEDX2",
        lineweight=5,
    )
    
    ml_builder.build(insert=Vec2(start_x_offset, start_y_offset))

def draw_section_view(form_data, doc = ezdxf.new(dxfversion='R2010'), x_offset=0, y_offset=0):
    
    msp = doc.modelspace()
    msp.add_blockref('TOWER_BLOCK', (x_offset, y_offset), dxfattribs={'layer': 'DIM', 'color': colors.LIGHT_GRAY})
    
    msp.add_lwpolyline(
        [(x_offset - 40, y_offset), ( x_offset + 100, y_offset)], 
        close=False, dxfattribs={'layer': 'DIM'}
    )
    
    tower_height = form_data['tower_height']
    tower_width = form_data['tower_width']
    hood_height = form_data['hood_height']
    hood_width = form_data['hood_width']
    plenum_depth = form_data['plenum_depth']
    # plenum_length = form_data['plenum_length']
    plenum_length = 8
    
    hood_information = form_data['HOOD_INFORMATION']
    hood_schedule_information = form_data['HOOD_SCHEDULE_INFO']
    
    exh_collar = hood_information[0]['EXH_COLLAR']
    exh_collar_qty = hood_information[0]['EXH_COLLAR_QTY']
    mua_collar = hood_information[0]['MUA_COLLAR']
    mua_collar_qty = hood_information[0]['MUA_COLLAR_QTY']
    
    collars = {
        'exh_collar': exh_collar,
        'exh_collar_qty': exh_collar_qty,
        'mua_collar': mua_collar,
        'mua_collar_qty': mua_collar_qty
    }
    
    plenum = {
        'plenum_depth': plenum_depth,
        'plenum_length': plenum_length
    }
    
    total_length = 0
    
    current_x_offset = x_offset + tower_width
    total_length += hood_width + plenum_depth
    
    current_y_offset = y_offset + tower_height * 0.8
    
    draw_section_view_hood(
        msp, 
        x_offset = current_x_offset, 
        y_offset = current_y_offset, 
        hood_length = hood_width, 
        hood_height = hood_height, 
        plenum = plenum, 
        hood_info=hood_schedule_information[0],
        collar=collars
    )
    
    
    draw_horizontal_dimension_line_right(
        msp, 
        x_offset=current_x_offset + hood_width, 
        y_offset=current_y_offset - hood_height, 
        dist=38,
        height=(tower_height * 0.8) - hood_height, 
        val=f"{(tower_height * 0.8) - hood_height} {inches_to_feet_and_inches(f"{int((tower_height * 0.8) - hood_height)}")}", 
        layer='HOOD-ACCESSORIES'
    )
    
    draw_vertical_dimension_line(
        msp,
        x_offset=current_x_offset, 
        y_offset=current_y_offset, 
        height=hood_height, 
        width=hood_width, 
        val_top=f"{hood_width}", 
        val_center=inches_to_feet_and_inches(f"{hood_width}"), 
        layer='HOOD-ACCESSORIES'
    )
    
    draw_section_view_plenum(
        msp, 
        x_offset = current_x_offset + hood_width, 
        y_offset = current_y_offset, 
        plenum_depth = plenum_depth, 
        plenum_length = plenum_length,
        layer='HOOD'
    )
    
    draw_vertical_dimension_line(
        msp,
        x_offset=current_x_offset + hood_width, 
        y_offset=current_y_offset - plenum_length, 
        height=hood_height - plenum_length, 
        width=plenum_depth, 
        from_height=True,
        val_top=f"{plenum_depth}",
        val_center=inches_to_feet_and_inches(f"{plenum_depth}"), 
        layer='HOOD-ACCESSORIES'
    )
    
    draw_horizontal_dimension_line_right(
        msp, 
        x_offset = current_x_offset + hood_width + plenum_depth, 
        y_offset = current_y_offset, 
        height = plenum_length, 
        dist = 12,
        val = f"{plenum_length}\"", 
        layer = 'HOOD-ACCESSORIES'
    )
    
    draw_horizontal_dimension_line_right(
        msp, 
        x_offset = current_x_offset + hood_width, 
        y_offset = current_y_offset, 
        height = hood_height, 
        dist = 45,
        val = f"{hood_height}\" {inches_to_feet_and_inches(f'{hood_height}')}", 
        layer = 'HOOD-ACCESSORIES'
    )
    
    draw_section_view_appliance(msp, x_offset = current_x_offset + 3, y_offset = y_offset + 6, layer='APPLIANCE')
    
    # draw_right_arrow(msp, base_x = current_x_offset + 3, base_y = y_offset + 6, end_x = current_x_offset + 10, end_y = y_offset + 16, width=1, height=2, layer='INDICATOR', direction='right')
    draw_section_view_info_label(msp, text="U.L. INCANDESCENT LIGHT", start_x_offset = current_x_offset - hood_height, start_y_offset = y_offset + tower_height - 10, pointer_x_offset=current_x_offset + hood_height, pointer_y_offset = current_y_offset - 5)
    draw_section_view_info_label(msp, text="HOOD TEMPERATURE SENSOR", start_x_offset = current_x_offset - hood_height, start_y_offset = y_offset + tower_height - 20, pointer_x_offset=current_x_offset + hood_height - 3, pointer_y_offset = current_y_offset - 5)
    draw_section_view_info_label(msp, text="3\" INTEGRAL AIR SPACE ON \n HOOD FOR 0\" CLEARANCE TO \n LIMITED COMBUSTIBLE SURFACES", start_x_offset = current_x_offset - hood_height, start_y_offset = y_offset + tower_height - 35, pointer_x_offset=current_x_offset + 3, pointer_y_offset = current_y_offset - 12)
    draw_section_view_info_label(msp, text="U.L CLASSIFIED BAFFLE FILTERS", start_x_offset = current_x_offset - hood_height, start_y_offset = y_offset + tower_height - 50, pointer_x_offset=current_x_offset + 10, pointer_y_offset = current_y_offset - 14)
    draw_section_view_info_label(msp, text="GREASE TROUGH \n WITH CONTAINER PER NFPA 96", start_x_offset = current_x_offset - hood_height, start_y_offset = y_offset + tower_height - 60, pointer_x_offset=current_x_offset + 7, pointer_y_offset = current_y_offset - 25)
    
    add_section_header(msp, text='SECTION VIEW', x_offset=x_offset + 50, y_offset=y_offset - 25, font_height=6)
    
# Section View END
